#include <ports.h>
#include <interrupt.h>
#include <video.h>
#include <stdlib.h>

idt_gate idt[256];
idtrc idtr = {2047, idt}; // 256*8-1
isr isrs[256];

void noint(registers* r) {
	*((uint16_t*)0xB8000) = 0x0F00 | r->ino;
}

void set_idt(uint8_t n, uint32_t i) {
	idt[n].hi = (i >> 16) & 0xFFFF;
	idt[n].lo = i & 0xFFFF;
	idt[n].sel = 0x8; // TODO macro
	idt[n].flags = 0x8E;
	idt[n].reserved = 0;
}

void lidt() {
	asm volatile("lidtl (%0)" : : "r" (&idtr));
}

void isr_init() {

	uint8_t portouts[20] = {
		0x20, 0x11,
		0xA0, 0x11,
		0x21, 0x20,
		0xA1, 0x28,
		0x21, 0x04,
		0xA1, 0x02,
		0x21, 0x01,
		0xA1, 0x01,
		0x21, 0x0,
		0xA1, 0x0
	};

	for(int i = 0; i < 20; i+=2) outb(portouts[i], portouts[i+1]); // remap pic TODO undo function

	for(int i = 0; i < 256; i++) { // TODO this can be done at compile time (even with relocation?)
		set_idt(i, ((uint32_t)isr_base) + (13*i));
		isrs[i] = noint;
	}

	lidt();
	asm volatile("sti");
}

void regint(uint8_t ino, isr i) {isrs[ino] = i;}

void isr_handler(registers* t){
	if(t->ino >= 32 && t->ino <= 47) { // from the pic
		if(t->ino >= 40) outb(0xA0, 0x20);
		outb(0x20, 0x20);
	}
	isrs[(uint8_t)t->ino](t);
}
